      SUBROUTINE CSTGLD(LAMF,MUF,ALF,GF,EMP,EFP,QFF)

C            CONFIGURATION MANAGEMENT OF EDF VERSION
C MODIF ELEMENTS  DATE 09/11/2012   AUTEUR DELMAS J.DELMAS 
C ======================================================================
C COPYRIGHT (C) 1991 - 2012  EDF R&D                  WWW.CODE-ASTER.ORG
C THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
C IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
C THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
C (AT YOUR OPTION) ANY LATER VERSION.
C
C THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
C WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
C MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
C GENERAL PUBLIC LICENSE FOR MORE DETAILS.
C
C YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
C ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
C   1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
C ======================================================================
C RESPONSABLE SFAYOLLE S.FAYOLLE

      IMPLICIT NONE
      REAL*8 LAMF,MUF,ALF,GF,TROT,TROT2
      REAL*8 EMP(2),EFP(2)

C----------------------------------------------------------------------
C        CALCUL DES CONSTANTES INDEPENDANTES DE DA1, DA2 ET EPS33
C
C     IN :
C         LAMBDA : PARAMETRE D ELASTICITE - MEMBRANE
C         MU     : PARAMETRE D ELASTICITE - MEMBRANE
C         LAMF   : PARAMETRE D ELASTICITE - FLEXION
C         MUF    : PARAMETRE D ELASTICITE - FLEXION
C         ALF    : PARAMETRE DE SEUIL FLEXION
C         GF     : PARAMETRE GAMMA POUR LA FLEXION
C         GMT    : PARAMETRE GAMMA POUR LA MEMBRANE EN TRACTION
C         GMC    : PARAMETRE GAMMA POUR LA MEMBRANE EN COMPRESSION
C         DELTA  : PARAMETRE DE COUPLAGE MEMBRANE-FLEXION
C         TROT   : TRACE DE KAPPA (FLEXION)
C         EMP(2) : VALEURS PROPRES DE EPS 2D
C         EFP(2) : VALEURS PROPRES DE KAPPA
C
C     OUT :
C         QFF(2) : TF
C         QM     : TM
C         COF1   : INTERMEDIAIRE DE CALCUL
C         Q2D    : INTERMEDIAIRE DE CALCUL
C         GI(2)  : PARTIE DE LA DERIVEE DE KSI(EMP) PAR RAPPORT A DA
C         GTR2   : INTERMEDIAIRE DE CALCUL
C         GI(2)  : INTERMEDIAIRE DE CALCUL
C----------------------------------------------------------------------

      INTEGER K
      REAL*8 QFF(2),GF1,GF2

C-- ICI ON SUPPOSE QUE GF1=GF2, CE QUI N EST PAS NECESSAIRE
      GF1 = GF
      GF2 = GF

      TROT = EFP(1) + EFP(2)
      TROT2 = TROT**2

C -------- CALCUL DE QFF --------------------

      IF(TROT .GT. 0.0D0) THEN
        QFF(1) = 0.0D0
        QFF(2) = 0.5D0*LAMF*TROT2
      ELSE
        QFF(1) = 0.5D0*LAMF*TROT2
        QFF(2) = 0.0D0
      ENDIF

      DO 4510, K = 1,2
        IF(EFP(K) .GT. 0.0D0) THEN
          QFF(2) = QFF(2) + MUF*EFP(K)**2
        ELSE
          QFF(1) = QFF(1) + MUF*EFP(K)**2
        ENDIF
4510  CONTINUE

      QFF(1) = ALF*QFF(1)*(1.0D0 - GF1)
      QFF(2) = ALF*QFF(2)*(1.0D0 - GF2)

      END
